From: "Aktemur, Tankut Baris" <tankut.baris.aktemur@intel.com>
To: Pedro Alves <palves@redhat.com>,
"gdb-patches@sourceware.org" <gdb-patches@sourceware.org>
Subject: RE: [PATCH 01/23] Preserve selected thread in all-stop w/ background execution
Date: Wed, 09 Oct 2019 09:36:00 -0000 [thread overview]
Message-ID: <B98F7326B8E238409968F562D326E1A90D076679@IRSMSX103.ger.corp.intel.com> (raw)
In-Reply-To: <20190906232807.6191-2-palves@redhat.com>
[-- Attachment #1: Type: text/plain, Size: 3872 bytes --]
* On September 7, 2019 1:28 AM, Pedro Alves wrote:
>
> diff --git a/gdb/infrun.c b/gdb/infrun.c
> index a9588f896a..9c888aa72f 100644
> --- a/gdb/infrun.c
> +++ b/gdb/infrun.c
> @@ -3048,6 +3048,11 @@ proceed (CORE_ADDR addr, enum gdb_signal siggnal)
>
> finish_state.release ();
>
> + /* If we've switched threads above, switch back to the previously
> + current thread. We don't want the user to see a different
> + selected thread. */
> + switch_to_thread (cur_thr);
> +
> /* Tell the event loop to wait for it to stop. If the target
> supports asynchronous execution, it'll do this from within
> target_resume. */
> @@ -3702,14 +3707,11 @@ fetch_inferior_event (void *client_data)
> set_current_traceframe (-1);
> }
>
> - gdb::optional<scoped_restore_current_thread> maybe_restore_thread;
> -
> - if (non_stop)
> - /* In non-stop mode, the user/frontend should not notice a thread
> - switch due to internal events. Make sure we reverse to the
> - user selected thread and frame after handling the event and
> - running any breakpoint commands. */
> - maybe_restore_thread.emplace ();
> + /* The user/frontend should not notice a thread switch due to
> + internal events. Make sure we revert to the user selected
> + thread and frame after handling the event and running any
> + breakpoint commands. */
> + scoped_restore_current_thread restore_thread;
>
Because this increases the refcount of the current thread, in case the
fetched inferior event denotes a thread exit, the thread will not be deleted
right away. A non-deleted but exited thread stays in the inferior's thread
list. This, in turn, causes the "init_thread_list" call in inferior.c to
be skipped. As a side effect, a regression is observed in
gdb.arch/i386-mpx-simple_segv.exp
IMHO, the 'any_thread_p' predicate should be updated. This predicate is used
in two places (one in 'inferior.c' and the other in 'mi/mi-main.c'). Both
uses, I believe, are in fact interested in whether there are any non-exited
threads. I'd suggest updating 'any_thread_p' to 'any_non_exited_thread_p'.
I'm attaching a patch that can be used to test.
> overlay_cache_invalid = 1;
> /* Flush target cache before starting to handle each event. Target
> @@ -3786,6 +3788,19 @@ fetch_inferior_event (void *client_data)
> inferior_event_handler (INF_EXEC_COMPLETE, NULL);
> cmd_done = 1;
> }
> +
> + /* If we got a TARGET_WAITKIND_NO_RESUMED event, then the
> + previously selected thread is gone. We have two
> + choices - switch to no thread selected, or restore the
> + previously selected thread (now exited). We chose the
> + later, just because that's what GDB used to do. After
> + this, "info threads" says "The current thread <Thread
> + ID 2> has terminated." instead of "No thread
> + selected.". */
> + if (!non_stop
> + && cmd_done
> + && ecs->ws.kind != TARGET_WAITKIND_NO_RESUMED)
> + restore_thread.dont_restore ();
> }
> }
>
The comment and the code seem to contradict each other. The comment says
"if we got a TARGET_WAITKIND_NO_RESUMED" whereas the condition is
ecs->ws.kind != TARGET_WAITKIND_NO_RESUMED
Should TARGET_WAITKIND_THREAD_EXITED, TARGET_WAITKIND_EXITED, and
TARGET_WAITKIND_SIGNALLED be also included in the condition? They also mean
that the thread is gone, right?
Regards,
-Baris
Intel Deutschland GmbH
Registered Address: Am Campeon 10-12, 85579 Neubiberg, Germany
Tel: +49 89 99 8853-0, www.intel.de
Managing Directors: Christin Eisenschmid, Gary Kershaw
Chairperson of the Supervisory Board: Nicole Lau
Registered Office: Munich
Commercial Register: Amtsgericht Muenchen HRB 186928
[-- Attachment #2: any_thread_p.patch --]
[-- Type: application/octet-stream, Size: 1974 bytes --]
diff --git a/gdb/inferior.c b/gdb/inferior.c
index 96d38f865d..9e254a96af 100644
--- a/gdb/inferior.c
+++ b/gdb/inferior.c
@@ -249,7 +249,7 @@ inferior_appeared (struct inferior *inf, int pid)
{
/* If this is the first inferior with threads, reset the global
thread id. */
- if (!any_thread_p ())
+ if (!any_non_exited_thread_p ())
init_thread_list ();
inf->pid = pid;
diff --git a/gdb/mi/mi-main.c b/gdb/mi/mi-main.c
index 5dc436bfd8..ee690e6daf 100644
--- a/gdb/mi/mi-main.c
+++ b/gdb/mi/mi-main.c
@@ -1991,7 +1991,7 @@ mi_execute_command (const char *cmd, int from_tty)
top_level_interpreter ()->interp_ui_out ()->is_mi_like_p ()
/* Don't try report anything if there are no threads --
the program is dead. */
- && any_thread_p ()
+ && any_non_exited_thread_p ()
/* If the command already reports the thread change, no need to do it
again. */
&& !command_notifies_uscc_observer (command.get ()))
diff --git a/gdb/thread.c b/gdb/thread.c
index 9dafca57bf..37e007d9c5 100644
--- a/gdb/thread.c
+++ b/gdb/thread.c
@@ -581,9 +581,9 @@ iterate_over_threads (int (*callback) (struct thread_info *, void *),
/* See gdbthread.h. */
bool
-any_thread_p ()
+any_non_exited_thread_p ()
{
- for (thread_info *tp ATTRIBUTE_UNUSED : all_threads ())
+ for (thread_info *tp ATTRIBUTE_UNUSED : all_non_exited_threads ())
return true;
return false;
}
next prev parent reply other threads:[~2019-10-09 9:36 UTC|newest]
Thread overview: 67+ messages / expand[flat|nested] mbox.gz Atom feed top
2019-09-06 23:28 [PATCH 00/23] Multi-target support Pedro Alves
2019-09-06 23:28 ` [PATCH 09/23] switch inferior/thread before calling target methods Pedro Alves
2019-09-06 23:28 ` [PATCH 20/23] Revert 'Remove unused struct serial::name field' Pedro Alves
2019-09-06 23:47 ` Christian Biesinger via gdb-patches
2019-09-08 19:30 ` Pedro Alves
2019-09-06 23:28 ` [PATCH 16/23] Fix reconnecting to a gdbserver already debugging multiple processes, II Pedro Alves
2019-09-06 23:28 ` [PATCH 03/23] Make "show remote exec-file" inferior-aware Pedro Alves
2019-09-06 23:28 ` [PATCH 19/23] gdbarch-selftests.c: No longer error out if debugging something Pedro Alves
2019-09-06 23:28 ` [PATCH 06/23] Don't check target is running in remote_target::mourn_inferior Pedro Alves
2019-09-06 23:28 ` [PATCH 17/23] Multi-target support Pedro Alves
2019-09-11 17:11 ` Tom Tromey
2019-10-17 1:54 ` Pedro Alves
2019-09-06 23:28 ` [PATCH 18/23] Add multi-target tests Pedro Alves
2019-10-09 16:01 ` Aktemur, Tankut Baris
2019-10-17 0:55 ` Pedro Alves
2019-09-06 23:28 ` [PATCH 08/23] Introduce switch_to_inferior_no_thread Pedro Alves
2019-09-09 18:42 ` Tom Tromey
2019-10-17 1:07 ` Pedro Alves
2019-09-06 23:28 ` [PATCH 11/23] tfile_target::close: trace_fd can't be -1 Pedro Alves
2019-09-06 23:28 ` [PATCH 13/23] Delete exit_inferior_silent(int pid) Pedro Alves
2019-09-06 23:28 ` [PATCH 02/23] Don't rely on inferior_ptid in record_full_wait Pedro Alves
2020-07-31 3:17 ` Tom Tromey
2020-08-01 16:14 ` Simon Marchi
2020-08-01 19:32 ` John Baldwin
2020-08-01 20:47 ` Tom Tromey
2020-08-01 20:46 ` Tom Tromey
2020-08-01 22:56 ` Simon Marchi
2020-08-02 17:52 ` Tom Tromey
2020-08-03 0:08 ` Simon Marchi
2019-09-06 23:28 ` [PATCH 01/23] Preserve selected thread in all-stop w/ background execution Pedro Alves
2019-10-09 9:36 ` Aktemur, Tankut Baris [this message]
2019-10-16 23:54 ` [PATCH v1.1 " Pedro Alves
2019-10-17 10:21 ` Aktemur, Tankut Baris
2019-09-06 23:28 ` [PATCH 10/23] Some get_last_target_status tweaks Pedro Alves
2019-09-09 18:53 ` Tom Tromey
2019-10-17 1:14 ` Pedro Alves
2019-09-06 23:28 ` [PATCH 15/23] Fix reconnecting to a gdbserver already debugging multiple processes, I Pedro Alves
2019-09-06 23:33 ` [PATCH 23/23] Multi-target: NEWS and user manual Pedro Alves
2019-09-07 6:33 ` Eli Zaretskii
2019-10-17 2:08 ` Pedro Alves
2019-10-17 7:55 ` Eli Zaretskii
2019-10-17 2:42 ` Pedro Alves
2019-10-17 8:14 ` Eli Zaretskii
2019-10-17 15:31 ` Pedro Alves
2019-09-06 23:34 ` [PATCH 04/23] exceptions.c:print_flush: Remove obsolete check Pedro Alves
2019-09-09 18:07 ` Tom Tromey
2019-09-06 23:35 ` [PATCH 05/23] Make target_ops::has_execution take an 'inferior *' instead of a ptid_t Pedro Alves
2019-09-09 18:12 ` Tom Tromey
2019-09-06 23:36 ` [PATCH 07/23] Delete unnecessary code from kill_command Pedro Alves
2019-10-01 10:19 ` Aktemur, Tankut Baris
2019-10-01 13:28 ` Aktemur, Tankut Baris
2019-09-06 23:36 ` [PATCH 14/23] Tweak handling of remote errors in response to resumption packet Pedro Alves
2019-10-09 13:35 ` Aktemur, Tankut Baris
2019-10-17 0:54 ` [PATCH 14.5/23] Avoid another inferior_ptid reference in gdb/remote.c (Re: [PATCH 14/23] Tweak handling of remote errors in response to resumption packet) Pedro Alves
2019-09-06 23:36 ` [PATCH 12/23] Use all_non_exited_inferiors in infrun.c Pedro Alves
2019-09-06 23:37 ` [PATCH 21/23] Add "info connections" command, "info inferiors" connection number/string Pedro Alves
2019-09-09 20:18 ` Tom Tromey
2019-10-17 2:21 ` Pedro Alves
2019-10-17 14:23 ` Tom Tromey
2019-09-06 23:37 ` [PATCH 22/23] Require always-non-stop for multi-target resumptions Pedro Alves
2019-09-07 11:19 ` [PATCH 00/23] Multi-target support Philippe Waroquiers
2019-09-08 20:06 ` Pedro Alves
2019-09-08 20:50 ` Philippe Waroquiers
2019-10-16 19:08 ` Pedro Alves
2019-10-16 19:14 ` [PATCH] Avoid inferior_ptid reference in gdb/remote.c (Re: [PATCH 00/23] Multi-target support) Pedro Alves
2019-09-09 19:09 ` [PATCH 00/23] Multi-target support Tom Tromey
2019-09-09 20:22 ` Tom Tromey
Reply instructions:
You may reply publicly to this message via plain-text email
using any one of the following methods:
* Save the following mbox file, import it into your mail client,
and reply-to-all from there: mbox
Avoid top-posting and favor interleaved quoting:
https://en.wikipedia.org/wiki/Posting_style#Interleaved_style
* Reply using the --to, --cc, and --in-reply-to
switches of git-send-email(1):
git send-email \
--in-reply-to=B98F7326B8E238409968F562D326E1A90D076679@IRSMSX103.ger.corp.intel.com \
--to=tankut.baris.aktemur@intel.com \
--cc=gdb-patches@sourceware.org \
--cc=palves@redhat.com \
/path/to/YOUR_REPLY
https://kernel.org/pub/software/scm/git/docs/git-send-email.html
* If your mail client supports setting the In-Reply-To header
via mailto: links, try the mailto: link
Be sure your reply has a Subject: header at the top and a blank line
before the message body.
This is a public inbox, see mirroring instructions
for how to clone and mirror all data and code used for this inbox
diff --git a/gdb/gdbthread.h b/gdb/gdbthread.h
index d00cc3fb8f..ce04fa6e42 100644
--- a/gdb/gdbthread.h
+++ b/gdb/gdbthread.h
@@ -579,8 +579,8 @@ all_threads_safe ()
extern int thread_count (process_stratum_target *proc_target);
-/* Return true if we have any thread in any inferior. */
-extern bool any_thread_p ();
+/* Return true if we have any non-exited thread in any inferior. */
+extern bool any_non_exited_thread_p ();
/* Switch context to thread THR. Also sets the STOP_PC global. */
extern void switch_to_thread (struct thread_info *thr);